@@ -42,8 +42,8 @@ import Bustle.Regions
 import Data.List.NonEmpty (NonEmpty(..))
 import qualified Data.Set as Set
 import Data.Set (Set)
-import qualified Data.Map as Map
-import Data.Map (Map)
+import qualified Data.Map.Strict as Map
+import Data.Map.Strict (Map)
 
 import Control.Arrow (first)
 import Control.Monad
@@ -59,10 +59,6 @@ data Bus = SessionBus
          | SystemBus
     deriving (Show, Eq, Ord)
 
-describeBus :: Bus -> String
-describeBus SessionBus = "session"
-describeBus SystemBus = "system"
-
 -- We keep the column in the map to allow the Monoid instance to preserve the
 -- ordering returned by sessionParticipants, which is the only view on this
 -- type exported.
@@ -142,8 +138,8 @@ instance Semigroup apps => Semigroup (RendererResult apps) where
 instance Monoid apps => Monoid (RendererResult apps) where
     mempty = RendererResult 0 0 [] [] mempty []
 
-processWithFilters :: (Log, Set UniqueName)
-                   -> (Log, Set UniqueName)
+processWithFilters :: (Log, NameFilter)
+                   -> (Log, NameFilter)
                    -> RendererResult ()
 processWithFilters (sessionBusLog, sessionFilter)
                    (systemBusLog,  systemFilter ) =
@@ -159,7 +155,7 @@ process sessionBusLog systemBusLog =
 
 -- Doesn't let you filter
 rendererStateNew :: RendererState
-rendererStateNew = initialState Set.empty Set.empty
+rendererStateNew = initialState emptyNameFilter emptyNameFilter
 
 buildResult :: RendererOutput
             -> RendererState
@@ -243,7 +239,8 @@ data BusState =
              , nextColumn :: Double
              , columnsInUse :: Set Double
              , pending :: Pending
-             , bsIgnoredNames :: Set UniqueName
+             , bsFilter :: NameFilter
+             , nextFakeName :: Integer
              }
 
 data RendererState =
@@ -254,7 +251,7 @@ data RendererState =
                   , startTime :: Microseconds
                   }
 
-initialBusState :: Set UniqueName
+initialBusState :: NameFilter
                 -> Double
                 -> BusState
 initialBusState ignore x =
@@ -263,17 +260,18 @@ initialBusState ignore x =
              , nextColumn = x
              , columnsInUse = Set.empty
              , pending = Map.empty
-             , bsIgnoredNames = ignore
+             , bsFilter = ignore
+             , nextFakeName = 0
              }
 
-initialSessionBusState, initialSystemBusState :: Set UniqueName -> BusState
+initialSessionBusState, initialSystemBusState :: NameFilter -> BusState
 initialSessionBusState f =
     initialBusState f $ timestampAndMemberWidth + firstColumnOffset
 initialSystemBusState f =
     initialBusState f $ negate firstColumnOffset
 
-initialState :: Set UniqueName
-             -> Set UniqueName
+initialState :: NameFilter
+             -> NameFilter
              -> RendererState
 initialState sessionFilter systemFilter = RendererState
     { sessionBusState = initialSessionBusState sessionFilter
@@ -359,18 +357,12 @@ lookupOtherName bus o = do
     case filter (Set.member o . aiCurrentNames . snd) (Map.assocs as) of
         [details] -> return details
 
-        -- No matches indicates a corrupt log, which we try to recover from …
+        -- No known owner for the well-known name. This happens in many cases,
+        -- especially when a method call causes service activation.
         []        -> do
-            warn $ concat [ "'"
-                          , unOtherName o
-                          , "' appeared unheralded on the "
-                          , describeBus bus
-                          , " bus; making something up..."
-                          ]
-            let namesInUse = Map.keys as
-                candidates = map (fakeUniqueName . show)
-                                 ([1..] :: [Integer])
-                u = head $ filter (not . (`elem` namesInUse)) candidates
+            n <- getsBusState nextFakeName bus
+            modifyBusState bus $ \bs -> bs { nextFakeName = n + 1 }
+            let u = fakeUniqueName (show n)
             addUnique bus u
             addOther bus o u
             ai <- lookupUniqueName bus u
@@ -648,9 +640,12 @@ shouldShow :: Bus
            -> Message
            -> Renderer Bool
 shouldShow bus m = do
-    ignored <- getsBusState bsIgnoredNames bus
+    nameFilter <- getsBusState bsFilter bus
     names <- mapM (fmap fst . lookupApp bus) (mentionedNames m)
-    return $ Set.null (ignored `Set.intersection` Set.fromList names)
+    return $
+        not (any (flip Set.member $ nfNever nameFilter) names)
+        && (Set.null (nfOnly nameFilter)
+            || any (flip Set.member $ nfOnly nameFilter) names)
 
 processOne :: Bus
            -> Detailed Event
