@@ -65,20 +65,12 @@ module Bustle.Types
   )
 where
 
-import Data.Word (Word32)
-import DBus ( ObjectPath, formatObjectPath
-            , InterfaceName, formatInterfaceName, interfaceName_
-            , MemberName, formatMemberName
-            , BusName, formatBusName, busName_
-            , ReceivedMessage
-            )
+import Bustle.GDBusMessage
 import Data.Maybe (maybeToList)
 import Data.Either (partitionEithers)
 import Data.Set (Set)
 import qualified Data.Set as Set
 
-type Serial = Word32
-
 newtype UniqueName = UniqueName BusName
   deriving (Ord, Show, Eq)
 newtype OtherName = OtherName BusName
@@ -195,7 +187,7 @@ data Detailed e =
     Detailed { deTimestamp :: Microseconds
              , deEvent :: e
              , deMessageSize :: MessageSize
-             , deReceivedMessage :: ReceivedMessage
+             , deReceivedMessage :: GDBusMessage
              }
   deriving (Show, Eq, Functor)
 
