@@ -41,6 +41,7 @@ import Bustle.Application.Monad
 import Bustle.Renderer
 import Bustle.Types
 import Bustle.Diagram
+import Bustle.GDBusMessage
 import qualified Bustle.Marquee as Marquee
 import Bustle.Monitor
 import Bustle.Util
@@ -284,9 +285,10 @@ recorderRun wi target filename r = C.handle newFailed $ do
     loaderStateRef <- newIORef Map.empty
     pendingRef <- newIORef []
 
-    let updateLabel µs body = do
+    let updateLabel :: Microseconds -> Int -> GDBusMessage -> IO ()
+        updateLabel µs l msg = do
             s <- readIORef loaderStateRef
-            (m, s') <- runStateT (convert µs body) s
+            (m, s') <- runStateT (convert µs l msg) s
             s' `seq` writeIORef loaderStateRef s'
 
             case m of
@@ -433,41 +435,6 @@ showSaveDialog wi savedCb = do
                             \\"%s\".") tempFilePath
                 displayError wi title (Just secondary)
 
--- | Show a confirmation dialog if the log is unsaved. Suitable for use as a
---   'delete-event' handler.
-promptToSave :: MonadIO io
-             => WindowInfo
-             -> io Bool -- ^ True if we showed a prompt; False if we're
-                        --   happy to quit
-promptToSave wi = io $ do
-    mdetails <- readIORef (wiLogDetails wi)
-    case mdetails of
-        Just (RecordedLog tempFilePath) -> do
-            let tempFileName = takeFileName tempFilePath
-                title = printf (__ "Save log '%s' before closing?") tempFileName :: String
-            prompt <- messageDialogNew (Just (wiWindow wi))
-                                       [DialogModal]
-                                       MessageWarning
-                                       ButtonsNone
-                                       title
-            messageDialogSetSecondaryText prompt
-                (__ "If you don't save, this log will be lost forever.")
-            dialogAddButton prompt (__ "Close _Without Saving") ResponseClose
-            dialogAddButton prompt stockCancel ResponseCancel
-            dialogAddButton prompt stockSave ResponseYes
-
-            widgetShowAll prompt
-            prompt `after` response $ \resp -> do
-                let closeUp = widgetDestroy (wiWindow wi)
-                case resp of
-                    ResponseYes -> showSaveDialog wi closeUp
-                    ResponseClose -> closeUp
-                    _ -> return ()
-                widgetDestroy prompt
-
-            return True
-        _ -> return False
-
 maybeQuit :: B ()
 maybeQuit = do
   n <- decWindows
@@ -487,7 +454,8 @@ emptyWindow = do
   subtitle <- getW castToLabel "headerSubtitle"
   spinner  <- getW castToSpinner "headerSpinner"
 
-  [openItem, openTwoItem] <- mapM (getW castToMenuItem) ["open", "openTwo"]
+  openItem <- getW castToMenuItem "open"
+  openTwoItem <- getW castToMenuItem "openTwo"
   recordSessionItem <- getW castToMenuItem "recordSession"
   recordSystemItem <- getW castToMenuItem "recordSystem"
   recordAddressItem <- getW castToMenuItem "recordAddress"
@@ -575,7 +543,6 @@ emptyWindow = do
                               , wiLogDetails = logDetailsRef
                               }
 
-  io $ window `on` deleteEvent $ promptToSave windowInfo
   incWindows
   io $ widgetShow window
   return windowInfo
