@@ -181,20 +181,14 @@ message_logged_cb (
     glong usec,
     guint8 *data,
     guint len,
+    GDBusMessage *message,
     gpointer user_data)
 {
-  g_autoptr(GError) error = NULL;
-  g_autoptr(GDBusMessage) message = g_dbus_message_new_from_blob (
-      data, len, G_DBUS_CAPABILITY_FLAGS_UNIX_FD_PASSING, &error);
-
-  if (message == NULL)
-    g_warning ("%s", error->message);
-  else
-    g_print ("%s -> %s: %d %s\n",
-        g_dbus_message_get_sender (message),
-        g_dbus_message_get_destination (message),
-        g_dbus_message_get_message_type (message),
-        g_dbus_message_get_member (message));
+  g_print ("%s -> %s: %d %s\n",
+      g_dbus_message_get_sender (message),
+      g_dbus_message_get_destination (message),
+      g_dbus_message_get_message_type (message),
+      g_dbus_message_get_member (message));
 }
 
 static void
