@@ -31,12 +31,12 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 
-#include <pcap/pcap.h>
-
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <gio/gunixinputstream.h>
 
+#include "pcap-reader.h"
+
 
 /* Prefix of name claimed by the connection that collects name owners. */
 const char *BUSTLE_MONITOR_NAME_PREFIX = "org.freedesktop.Bustle.Monitor.";
@@ -99,13 +99,12 @@ typedef struct _BustlePcapMonitor {
     GSubprocess *dbus_monitor;
     /* If >= 0, master side of controlling terminal for dbus_monitor */
     int pt_master;
-    GSource *dbus_monitor_source;
-    pcap_t *pcap_in;
+    GSubprocess *tee_proc;
+    GSource *tee_source;
+    BustlePcapReader *reader;
 
     /* output */
     gchar *filename;
-    pcap_t *pcap_out;
-    pcap_dumper_t *dumper;
 
     /* errors */
     GError *pcap_error;
@@ -205,16 +204,6 @@ bustle_pcap_monitor_set_property (
     }
 }
 
-static void
-close_dump (BustlePcapMonitor *self)
-{
-  if (self->dumper != NULL)
-    pcap_dump_flush (self->dumper);
-
-  g_clear_pointer (&self->dumper, pcap_dump_close);
-  g_clear_pointer (&self->pcap_out, pcap_close);
-}
-
 static void
 bustle_pcap_monitor_dispose (GObject *object)
 {
@@ -229,12 +218,11 @@ bustle_pcap_monitor_dispose (GObject *object)
     }
 
   g_clear_object (&self->cancellable);
-  g_clear_pointer (&self->dbus_monitor_source, g_source_destroy);
-  g_clear_pointer (&self->pcap_in, pcap_close);
+  g_clear_pointer (&self->tee_source, g_source_destroy);
+  g_clear_object (&self->tee_proc);
+  g_clear_object (&self->reader);
   g_clear_object (&self->dbus_monitor);
 
-  close_dump (self);
-
   if (parent_class->dispose != NULL)
     parent_class->dispose (object);
 }
@@ -300,15 +288,17 @@ bustle_pcap_monitor_class_init (BustlePcapMonitorClass *klass)
    *  #GValue.)
    * @blob: an array of bytes containing the serialized message.
    * @length: the size in bytes of @blob.
+   * @message: @blob as a #GDBusMessage.
    */
   signals[SIG_MESSAGE_LOGGED] = g_signal_new ("message-logged",
       BUSTLE_TYPE_PCAP_MONITOR, G_SIGNAL_RUN_FIRST,
       0, NULL, NULL,
-      NULL, G_TYPE_NONE, 4,
+      NULL, G_TYPE_NONE, 5,
       G_TYPE_LONG,
       G_TYPE_LONG,
       G_TYPE_POINTER,
-      G_TYPE_UINT);
+      G_TYPE_UINT,
+      G_TYPE_DBUS_MESSAGE);
 
   /**
    * BustlePcapMonitor::stopped:
@@ -405,7 +395,6 @@ handle_error (BustlePcapMonitor *self)
     }
 
   self->state = STATE_STOPPED;
-  close_dump (self);
 
   g_debug ("%s: emitting ::stopped(%s, %d, %s)", G_STRFUNC,
            g_quark_to_string (error->domain), error->code, error->message);
@@ -652,9 +641,8 @@ start_pcap (
   GInputStream *stdout_pipe = NULL;
   gint stdout_fd = -1;
   FILE *dbus_monitor_filep = NULL;
-  char errbuf[PCAP_ERRBUF_SIZE] = {0};
 
-  stdout_pipe = g_subprocess_get_stdout_pipe (self->dbus_monitor);
+  stdout_pipe = g_subprocess_get_stdout_pipe (self->tee_proc);
   g_return_val_if_fail (stdout_pipe != NULL, FALSE);
 
   stdout_fd = g_unix_input_stream_get_fd (G_UNIX_INPUT_STREAM (stdout_pipe));
@@ -674,24 +662,17 @@ start_pcap (
    * fread(). It's safe to do this on the main thread, since we know the pipe
    * is readable. On short read, pcap_fopen_offline() fails immediately.
    */
-  self->pcap_in = pcap_fopen_offline (dbus_monitor_filep, errbuf);
-  if (self->pcap_in == NULL)
+  self->reader = bustle_pcap_reader_fopen (g_steal_pointer (&dbus_monitor_filep), error);
+  if (self->reader == NULL)
     {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-                   "Couldn't read messages from dbus-monitor: %s",
-                   errbuf);
+      g_prefix_error (error, "Couldn't read messages from dbus-monitor: ");
 
-      /* Cause dbus-monitor to exit next time it tries to write a message */
-      g_clear_pointer (&dbus_monitor_filep, fclose);
-
-      /* And try to terminate it immediately. */
+      /* Try to terminate dbus-monitor immediately. The reader closes the FILE * on error. */
       send_sigint (self);
 
       return FALSE;
     }
 
-  /* pcap_close() will call fclose() on the FILE * passed to
-   * pcap_fopen_offline() */
   dump_names_async (self);
   self->state = STATE_RUNNING;
   return TRUE;
@@ -702,35 +683,28 @@ read_one (
     BustlePcapMonitor *self,
     GError **error)
 {
-  struct pcap_pkthdr *hdr;
+  glong sec, usec;
   const guchar *blob;
-  int ret;
+  guint length;
+  g_autoptr(GDBusMessage) message = NULL;
 
-  ret = pcap_next_ex (self->pcap_in, &hdr, &blob);
-  switch (ret)
+  if (!bustle_pcap_reader_read_one (self->reader, &sec, &usec, &blob, &length, &message, error))
     {
-      case 1:
-        g_signal_emit (self, signals[SIG_MESSAGE_LOGGED], 0,
-            hdr->ts.tv_sec, hdr->ts.tv_usec, blob, hdr->caplen);
+      return FALSE;
+    }
+  else if (message == NULL)
+    {
+      /* EOF; shouldn't happen since we waited for the FD to be readable */
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_CONNECTION_CLOSED,
+          "EOF when reading from dbus-monitor");
+      return FALSE;
+    }
+  else
+    {
+      g_signal_emit (self, signals[SIG_MESSAGE_LOGGED], 0,
+          sec, usec, blob, length, message);
 
-        /* cast necessary because pcap_dump has a type matching the callback
-         * argument to pcap_loop()
-         * TODO don't block
-         */
-        pcap_dump ((u_char *) self->dumper, hdr, blob);
-        return TRUE;
-
-      case -2:
-        /* EOF; shouldn't happen since we waited for the FD to be readable */
-        g_set_error (error, G_IO_ERROR, G_IO_ERROR_CONNECTION_CLOSED,
-            "EOF when reading from dbus-monitor");
-        return FALSE;
-
-      default:
-        g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-            "Error %i reading dbus-monitor stream: %s",
-            ret, pcap_geterr (self->pcap_in));
-        return FALSE;
+      return TRUE;
     }
 }
 
@@ -824,7 +798,7 @@ cancellable_cancelled_cb (GCancellable *cancellable,
   /* Closes the stream; should cause dbus-monitor to quit in due course when it
    * tries to write to the other end of the pipe.
    */
-  g_clear_pointer (&self->pcap_in, pcap_close);
+  bustle_pcap_reader_close (self->reader);
 
   /* And try to terminate it immediately. */
   send_sigint (self);
@@ -931,6 +905,27 @@ spawn_monitor (BustlePcapMonitor *self,
   return child;
 }
 
+static GSubprocess *
+spawn_tee (BustlePcapMonitor  *self,
+           GError            **error)
+{
+  g_autoptr(GSubprocessLauncher) launcher =
+    g_subprocess_launcher_new (G_SUBPROCESS_FLAGS_STDOUT_PIPE);
+  GInputStream *stdout_pipe = NULL;
+  gint stdout_fd = -1;
+
+  stdout_pipe = g_subprocess_get_stdout_pipe (self->dbus_monitor);
+  g_return_val_if_fail (stdout_pipe != NULL, FALSE);
+
+  stdout_fd = g_unix_input_stream_get_fd (G_UNIX_INPUT_STREAM (stdout_pipe));
+  g_return_val_if_fail (stdout_fd >= 0, FALSE);
+
+  g_subprocess_launcher_take_stdin_fd (launcher, stdout_fd);
+
+  return g_subprocess_launcher_spawn (launcher, error,
+                                      "tee", self->filename, NULL);
+}
+
 static gboolean
 initable_init (
     GInitable *initable,
@@ -957,36 +952,24 @@ initable_init (
                            G_CALLBACK (cancellable_cancelled_cb),
                            self, NULL);
 
-  self->pcap_out = pcap_open_dead (DLT_DBUS, 1 << 27);
-  if (self->pcap_out == NULL)
-    {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-          "pcap_open_dead failed. wtf");
-      return FALSE;
-    }
-
-  self->dumper = pcap_dump_open (self->pcap_out, self->filename);
-  if (self->dumper == NULL)
-    {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-          "Couldn't open target file %s", pcap_geterr (self->pcap_out));
-      return FALSE;
-    }
-
   self->dbus_monitor = spawn_monitor (self, (const char * const *) argv, error);
   if (self->dbus_monitor == NULL)
     return FALSE;
 
-  stdout_pipe = g_subprocess_get_stdout_pipe (self->dbus_monitor);
+  self->tee_proc = spawn_tee (self, error);
+  if (self->tee_proc == NULL)
+    return FALSE;
+
+  stdout_pipe = g_subprocess_get_stdout_pipe (self->tee_proc);
   g_return_val_if_fail (stdout_pipe != NULL, FALSE);
   g_return_val_if_fail (G_IS_POLLABLE_INPUT_STREAM (stdout_pipe), FALSE);
   g_return_val_if_fail (G_IS_UNIX_INPUT_STREAM (stdout_pipe), FALSE);
 
-  self->dbus_monitor_source = g_pollable_input_stream_create_source (
+  self->tee_source = g_pollable_input_stream_create_source (
       G_POLLABLE_INPUT_STREAM (stdout_pipe), self->cancellable);
-  g_source_set_callback (self->dbus_monitor_source,
+  g_source_set_callback (self->tee_source,
       (GSourceFunc) dbus_monitor_readable, self, NULL);
-  g_source_attach (self->dbus_monitor_source, NULL);
+  g_source_attach (self->tee_source, NULL);
 
   g_subprocess_wait_check_async (
       self->dbus_monitor,
